# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""This script defines a custom transformer."""

import io
import base64
import argparse

from typing import Dict

import kserve

from PIL import Image
from torchvision import transforms


DEFAULT_MODEL_NAME = "custom-model"
DEFAULT_PROTOCOL_VERSION = "v1"


class Transformer(kserve.Model):
    """Define custom transformer."""

    def __init__(self, model_name: str, predictor_host: str, protocol: str):
        super().__init__(model_name)
        self.predictor_host = predictor_host
        self.protocol = protocol

    def preprocess(self, request: Dict):
        inputs = request["instances"]
        data = inputs[0]["image"]["b64"]

        raw_img_data = base64.b64decode(data)
        input_image = Image.open(io.BytesIO(raw_img_data))

        preprocess = transforms.Compose([
            transforms.Resize(256),
            transforms.CenterCrop(224),
            transforms.ToTensor(),
            transforms.Normalize(mean=[0.485, 0.456, 0.406],
                                 std=[0.229, 0.224, 0.225]),
        ])

        input_tensor = preprocess(input_image)
        return {"instances": [input_tensor.numpy().tolist()]}

    def postprocess(self, request: Dict) -> Dict:
        return request


if __name__ == "__main__":
    parser = argparse.ArgumentParser(parents=[kserve.model_server.parser])
    parser.add_argument('--model_name', default=DEFAULT_MODEL_NAME,
                        help='The name that the model is served under.')
    parser.add_argument('--predictor_host',
                        help='The URL for the model predict function',
                        required=True)
    parser.add_argument('--protocol', default="DEFAULT_PROTOCOL_VERSION",
                        help='The serving protocol for the predictor')
    args, _ = parser.parse_known_args()

    server = kserve.ModelServer()
    model = Transformer(model_name=args.model_name,
                        predictor_host=args.predictor_host,
                        protocol=args.protocol)
    server.start(models=[model])
